#include "libusart.h"
#include "libwait.h"

unsigned char usartStatus;
unsigned char usartTimeout;

unsigned char usartRbuf[USART_RBUF];
unsigned char usartRbrp;
unsigned char usartRbwp;

/*------------------------------------------------------------------------------
 *
 *	USART 初期設定
 *
 */
void initUsart() {
									// Baud Rate = FOSC / (16 * (X + 1))
									// 9615.38 = 4,000,000 / (16 * (25 + 1))

//	SPBRGH	= 0;					// Baud Rate Generator(High Byte)
	SPBRG	= BAUDRATE;				// Baud Rate Generator
	BRGH	= 1;					// High Baud Rate Select(High/Low)
//	BRG16	= 0;					// 16-Bit Baud Rate Register Enable bit
	SYNC	= 0;					// USART Mode Select(Synchronous/Asynchronous)

	SPEN	= 1;					// Serial Port Enable
	RCIE	= USART_INTR;			// USART Receive Interrupt Enable
	CREN	= 1;					// Continuous Receive Enable

	TXEN	= 1;					// Transmit Enable

	usartRbwp = usartRbrp = 0;		// Initialize Ring Buffer Pointer
}

/*------------------------------------------------------------------------------
 *
 *	USART 送信処理
 *
 */
void transmitUsart(unsigned char c) {

		while(!TRMT);
		TXREG = c;								// USART Transmit data register
}

/*------------------------------------------------------------------------------
 *
 *	USART 受信処理
 *
 */
unsigned char receiveUsart() {					// データ受信までブロックされる

	unsigned char recv;

	while(1) {
		while(!RCIF);							// 受信フラグチェック

		if(OERR) {								// オーバランエラーチェック
			CREN = 0; CREN = 1;					// 受信回路リセット(OERRクリア)
		}
		if(FERR) {								// フレーミングエラーチェック
			recv = RCREG;						// ゴミを吸い取る
			continue;
		}
		break;
	}
	recv = RCREG;								// エラーなし、受信データを得る 
	return(recv);
}

/*------------------------------------------------------------------------------
 *
 *	USART 受信処理(ノンブロック)
 *
 */
unsigned char receiveUsartNb() {

	unsigned char recv;

	if(!RCIF)									// 受信フラグチェック
		return(0x39);

	while(1) {
		if(OERR) {								// オーバランエラーチェック
			CREN = 0; CREN = 1;					// 受信回路リセット(OERRクリア)
		}
		if(FERR) {								// フレーミングエラーチェック
			recv = RCREG;						// ゴミを吸い取る
			return(0x38);
		}
		break;
	}
	recv = RCREG;								// エラーなし、受信データを得る 
	return(recv);
}

/*------------------------------------------------------------------------------
 *
 *	USART 受信処理(タイムアウト付き)
 *
 *	usartTimeout = 195;							// 4MHz 時、1秒の場合、195を与えると
 *	c = receiveUsartTo();						// 2行で 195 x 5120 + 50 = 998450us まで待つ
 *	if(!usartStatus) {
 */
unsigned char receiveUsartTo() {

	unsigned int rest;
	unsigned char recv;

	usartStatus = 0;
	for(rest = usartTimeout << 8; !RCIF && rest > 0; --rest);
	if(!rest) {									// タイムアウトチェック
		usartStatus |= 0b00000001;
		return(0xFF);
	}
	if(OERR) {									// オーバランエラーチェック
		CREN = 0; CREN = 1;						// 受信回路リセット(OERRクリア)
		usartStatus |= 0b00001000;
	}
	if(FERR) {									// フレーミングエラーチェック
		usartStatus |= 0b00000100;
	}
	recv = RCREG;								// 受信データを得る 
	return(recv);
}

/*------------------------------------------------------------------------------
 *
 *	USART 割り込み受信処理
 *
 *	void Intr(void) __interrupt 0 {
 *		receiveUsartIntr();
 *	}
 */
void receiveUsartIntr() {

	while(RCIF) {								// 受信フラグチェック
		if(OERR) {								// オーバランエラーチェック
			CREN = 0; CREN = 1;					// 受信回路リセット(OERRクリア)
			continue;
		}
		usartRbuf[usartRbwp] = RCREG;			// 受信データを得る 
		if(!FERR)								// フレーミングエラーチェック
			usartRbwp++;
		usartRbwp %= USART_RBUF;
	}
}

