#include "libxbeeapiframe.h"
#include "libusart.h"

int xbee_buf_txlen;
unsigned char xbee_buf_txsum;
unsigned char xbee_buf_tx[] = {
	XBEE_START_DLM,												// Start Delimiter
	0x00, 0x00,													// Length
	XBEE_XMIT_REQ,												// Frame Type
	0x80,														// Frame ID
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF,				// 64-bit Destination Address
	0xFF, 0xFE,													// 16-bit Destination Network Address
	0x00,														// Broadcast Radius
	0x00,														// Options
};

unsigned char xbee_buf_rxp, xbee_buf_rxsum;
unsigned char xbee_buf_rx[32];									// TODO: define, 32 = head:18 + dat:14

/*------------------------------------------------------------------------------
 *
 *	XBee API Frame 初期化
 *
 */
void xbee_API_Frame_init() {

	usartTimeout = 195;
	xbee_buf_rxp = 0;
}

/*------------------------------------------------------------------------------
 *
 *	XBee API Frame 文字列送信
 *
 *	xbee_buf_txlen = 8;
 *	xbee_API_Frame_puts("ABCDEFGH");
 */
void xbee_API_Frame_puts(unsigned char *s) {

	unsigned char p = 0;

	xbee_API_Frame_begin();
	while(xbee_buf_txlen > 0)
		xbee_API_Frame_putc(s[p++]);
	xbee_API_Frame_end();
}

/*------------------------------------------------------------------------------
 *
 *	XBee API Frame 開始
 *
 */
void xbee_API_Frame_begin() {

	unsigned char p;

	xbee_buf_tx[1] = (xbee_buf_txlen + 14) >> 8;
	xbee_buf_tx[2] = (xbee_buf_txlen + 14) & 0xFF;

	for(p = 0; p < 17; ) {										// Options まで 17 バイト送信
		transmitUsart(xbee_buf_tx[p]);
		xbee_buf_txsum += xbee_buf_tx[p++];

		if(p == 3)												// Length まで送信したらチェックサム加算を開始
			xbee_buf_txsum = 0;
	}
}

/*------------------------------------------------------------------------------
 *
 *	XBee API Frame 1 バイト送信
 *
 */
void xbee_API_Frame_putc(unsigned char c) {

	if(xbee_buf_txlen-- > 0) {
		transmitUsart(c);
		xbee_buf_txsum += c;
	} else {													// Auto Closing
		xbee_buf_txlen++;
		xbee_API_Frame_end();
	}
}

/*------------------------------------------------------------------------------
 *
 *	XBee API Frame 終了
 *
 */
void xbee_API_Frame_end() {

	while(xbee_buf_txlen-- > 0) {								// Auto Padding
		transmitUsart(0xFF);
		xbee_buf_txsum += 0xFF;
	}
	if(xbee_buf_txlen == -1)
		transmitUsart(0xFF - xbee_buf_txsum);
}

/*------------------------------------------------------------------------------
 *
 *	XBee API Frame 受信(割り込み受信リングバッファから)
 *
 *	struct xbee_API_Frame_RECV_PCKT *xbuf_rp = (struct xbee_API_Frame_RECV_PCKT*)xbee_buf_rx;
 *	while(1) {
 *		while((r = xbee_API_Frame_receive_rb()) == 1);			// フレーム受信 / 1: フレーム受信中
 *			if(r == 0) {										// 0: フレーム受信成功
 *				switch(xbuf_rp->type) {
 *				  case XBEE_RECV_PCKT:
 *					 :
 */
unsigned char xbee_API_Frame_receive_rb() {

	unsigned char c;

	if(usartRbrp == usartRbwp)
		return(3);												// バッファが空

	c = usartRbuf[usartRbrp++];
	usartRbrp %= USART_RBUF;

	if(xbee_buf_rxp) {											// フレーム受信中
		xbee_buf_rx[xbee_buf_rxp++] = c;
		xbee_buf_rxsum += c;

		if(xbee_buf_rxp == 3) {									// Length まで受信したらチェックサム加算を開始
			xbee_buf_rxsum = 0;

		} else if(xbee_buf_rxp == xbee_buf_rx[2] + 4) {			// チェックサムまで受信
			xbee_buf_rxp = 0;									// フレーム受信完了
			return(xbee_buf_rxsum == 0xFF ? 0 : 2);
		}

	} else if(c == XBEE_START_DLM) {							// Start Delimiter 検出
		xbee_buf_rx[xbee_buf_rxp++] = c;						// フレーム受信開始
		xbee_buf_rx[2] = 0xFF;									// Length 仮設定
	}

	return(1);
}

