#include <sdcc-lib.h>
#include <pic16f688.h>	// /usr/share/sdcc/non-free/include/pic14/pic16f688.h, /usr/share/sdcc/include/pic14/pic16f688.h

#include "inline.h"
#include "libwait.h"
#include "libusart.h"
#include "libxbeeapiframe.h"

static __code unsigned char __at (0x2000) id_location[] = {'A', 'B', 'C', 'D'};

#ifndef _BOR_ON
#define _BOR_ON		_BOD_ON
#define _BOR_OFF	_BOD_OFF
#endif

static __code int __at (0x2007) configword1 =
	_FCMEN_OFF &
	_IESO_OFF &
	_BOR_OFF &	// for 2.1V
	_CPD_OFF &
	_CP_OFF &
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

static __code unsigned char __at (0x2100) eedata[] = {'d', 'u', 'm', 'm', 'y', 0};

#define	STATLED		RA1
#define	MODESW		RA2
#define	XBSLEEP		RA4
#define	DISCHARGE	RC2				// L で放電
#define	CHARGE		RC3				// H で充電

unsigned char jiffies = 0;			// 15.3Hz 周期で加算
unsigned char trigger = 0;			// 16.8sec 周期でオン
unsigned char lastsw = 1;
unsigned char swpushed = 0;

void Intr(void) __interrupt 0 {
	receiveUsartIntr();

	if(T0IF) {
		T0IF = 0;					// Clear timer interrupt
		if(!(++jiffies))
			trigger = 1;
	}

	if((MODESW != lastsw) && !(lastsw = MODESW))
		swpushed = 1;
}

/*			  _______				PIC16F688
	Vcc		-|	 U	 |-	GND
	NC		-|5		0|-	BATVOLT
	XBSLEEP	-|4	 A	1|-	STATLED
	NC(GND)	-|3		2|-	MODESW
	RX		-|5		0|-	NC
	TX		-|4	 C	1|-	NC
	CHARGE	-|3_____2|-	DISCHARGE
*/

void main(void) {

	unsigned char mode;
	unsigned int stimer;
	unsigned char nflush;
	unsigned char volts;

	struct xbee_API_Frame_XMIT_REQ		*xbuf_xr = (struct xbee_API_Frame_XMIT_REQ*)xbee_buf_tx;
	struct xbee_API_Frame_MODEM_STAT	*xbuf_md = (struct xbee_API_Frame_MODEM_STAT*)xbee_buf_rx;
	struct xbee_API_Frame_XMIT_STAT		*xbuf_xs = (struct xbee_API_Frame_XMIT_STAT*)xbee_buf_rx;
	struct xbee_API_Frame_RECV_PCKT		*xbuf_rp = (struct xbee_API_Frame_RECV_PCKT*)xbee_buf_rx;

				//  76543210
	PORTA		= 0b00000100;		// RA3: input only
	CMCON0		= 0b00000111;
	ANSEL		= 0b00000001;
	TRISA		= 0b00001101;		// 1: 入力, 0: 出力
	WPUA		= 0b00000100;		// 初期値:0b00110111

	PORTC		= 0b00000100;
	TRISC		= 0b00000000;		// 1: 入力, 0: 出力

	OPTION_REG	= 0b01010111;		// 初期値:0b11111111
									// 7:PORTAプルアップ(0:使用する)
									// 6:RA2割り込みポート変化選択(0:立下り)
									// 5:TMR0クロックソース選択ビット(0:内部)
									// 4:TMR0加算タイミング(0:立上り)
									// 3:プリスケーラ割り当てビット(0:TMR0)
									// 2~0プリスケーラレート選択ビット(111:1/256)
									// 4MHz => 1MHz = 1,000,000 / 256 / 256 = 15Hz

	INTCON		= 0b10100000;		// 初期値:0b0000000x
									// 7:GIE  Global Interrupt Enable bit
									// 6:PEIE Peripheral Interrupt Enable bit
									// 5:T0IE TMR0 Overflow Interrupt Enable bit
									// 4:INTE RA2/INT External Interrupt Enable bit
									// 3:RAIE RA Port Change Interrupt Enable bit
									// 2:T0IF TMR0 Overflow Interrupt Flag bit
									// 1:INTF RA2/INT External Interrupt Flag bit
									// 0:RAIF RA Port Change Interrupt Flag bit

	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
									// 7:EEIE EE Write Complete Interrupt Enable Bit
									// 6:ADIE A/D Converter Interrupt Enable bit
									// 5:RCIE USART Receive Interrupt Enable bit
									// 4:C2IE Comparator 2 Interrupt Enable bit
									// 3:C1IE Comparator 1 Interrupt Enable bit
									// 2:OSFIE Oscillator Fail Interrupt Enable bit
									// 1:TXIE EUSART Transmit Interrupt Enable bit
									// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

	T1CON		= 0b00000000;		// 初期値:0b00000000
									// 7:T1GINV Timer1 Gate Invert bit
									// 6:TMR1GE Timer1 Gate Enable bit
									// 5~4:T1CKPS Timer1 Input Clock Prescale Select bits(00:1/1, 11:1/8)
									// 3:T1OSCEN Timer1 Oscillator Enable Control bit(1:Enable)
									// 2:T1SYNC Timer1 External Clock Input Synchronization Control bit(1:Sync)
									// 1:TMR1CS Timer1 Clock Source Select bit(0:Internal)
									// 0:TMR1ON Timer1 On bit(1:Enable)

	OSCTUNE = 0b00011110;			// fast 0b01111 <-> 0b10000 slow
//	forCalibOsc();
	initUsart();
	xbee_API_Frame_init();

	//--------------------------------------------------------------------------
	//
	//	Main Loop
	//
	ADCON1 = 0b01010000;										// TAD = 101: FOSC/16
	ADCON0 = 0b00000001;										// Left, VDD, AN0, enabled

	waitL(5);

	mode = 0;													// 0: 計測モード
	stimer = 6;													// 5 回ほど計測してから放電開始
	nflush = 1;													// 状態 LED 連続点滅

	while(1) {

		if(trigger || swpushed) {
			trigger = 0;

			if(!(--stimer))										// 予定の時間が経過したら、スイッチを押す
				swpushed = 1;

			if(swpushed) {
				swpushed = 0;
																// stimer 1h: 215, 4h: 858, 5h: 1073, 6h: 1287
				switch(mode) {

				  case 0:										// 0: 計測モード
					DISCHARGE = 0;								// 放電開始
					stimer = 10;								// 基準電圧未満を 10 回計測で放電終了
					mode += 2;
					break;

				  case 2:										// 2: 放電モード
					DISCHARGE = 1;								// 放電終了
					stimer = 215;								// 1時間休息
					++mode;
					nflush = 28;
					break;

				  case 3:										// 3: 休息モード
					CHARGE = 1;									// 充電開始
					stimer = 1073;								// 5時間充電
					++mode;
					break;

				  case 4:										// 4: 充電モード
					CHARGE = 0;									// 充電終了
					stimer = 215;								// 1時間完了(?)
					++mode;
					nflush = 30;
					break;

				  case 5:										// 5: 完了モード
					STATLED = 0;
					SLEEP();									// 完了終了(?)
				//	^^^^^^^^ HALT
					break;
				}
			}

			GO = 1;												// 電圧を測定
			while(GO);
			volts = ADRESH;

			//----------------------------------
			//
			//	状態の無線送信
			//
			XBSLEEP = 0;
//			waitM(4);											// 4: 2ms (経験的設定, 1.5ms でも OK)
			waitM(40);

			++xbuf_xr->id; xbuf_xr->id |= 0x80;
			xbee_buf_txlen = 5;
			xbee_API_Frame_begin();
			xbee_API_Frame_putc(0x22);							// 情報種別コード
			xbee_API_Frame_putc(mode);
			xbee_API_Frame_putc(volts);							// 電圧を送信
			xbee_API_Frame_putc(stimer >> 8);
			xbee_API_Frame_putc(stimer & 0xFF);
			xbee_API_Frame_end();

			waitM(200);											// 200: 100ms (経験的設定)
			waitM(200);
																// TODO: 各種コマンドを受け付ける
			XBSLEEP = 1;

			//----------------------------------
			//
			//	LED による電圧表現
			//
			if(!(mode & 0b00000001)) {							// 0: 計測モード / 2: 放電モード / 4: 充電モード
				do {
					  nflush = 5;								//   7 times
					if(volts > 0xA2) break;						// volts > 8.50 V
					  nflush = 9;								//   6 times
					if(volts > 0x99) break;						// volts > 8.00 V
					  nflush = 13;								//   5 times
					if(volts > 0x8F) break;						// volts > 7.50 V
					  nflush = 17;								//   4 times
					if(volts > 0x86) break;						// volts > 7.00 V
					  nflush = 21;								//   3 times
					if(volts > 0x7C) break;						// volts > 6.50 V
					  nflush = 25;								//   2 times
					if(volts > 0x72) break;						// volts > 6.00 V
					  nflush = 29;								//   1 times
				} while(0);

				if((mode == 2) && (volts > 0x72))				// 2: 放電モード && volts > 6.00 V
					++stimer;									// 時間を戻す
			}
		}

		//--------------------------------------
		//
		//	状態 LED 点滅
		//
		if(!(mode & 0b00000001)) {								// 0: 計測モード / 2: 放電モード / 4: 充電モード

			//      5   9  13  17  21  25  29
			// ------**--**--**--**--**--**--**

			STATLED = (jiffies & 0b00000010) && (nflush < (jiffies & 0b00011111));

		} else {												// 3: 休息モード / 5: 完了モード

			//                            2830
			// -----------------------------*-*

			STATLED = (jiffies & 0b00000001) && (nflush < (jiffies & 0b00011111));
		}
	}
}

